# -*- coding: utf-8 -*-

import os
import oss2
import hashlib
from config import config


class Oss(object):

    def __init__(self):
        config_name = os.environ.get('FLASK_CONFIG', 'dev')
        conf = config[config_name]
        self.conf = conf
        self.auth = oss2.Auth(conf.OSS_CONF['access_key_id'], conf.OSS_CONF['access_key_secret'])
        self.bucket = oss2.Bucket(self.auth, conf.OSS_CONF['endpoint'], conf.OSS_CONF['bucket'])

        # 设置允许上传的文件类型
        self.file_type_txt = ['text/plain']
        self.file_type_img = ['image/jpeg', 'image/bmp', 'image/png', 'image/gif']
        self.file_type_pdf = ['application/pdf']
        self.file_type_doc = ['application/msword', 'application/vnd.openxmlformats-officedocument.wordprocessingml.document']
        self.file_type_xls = ['application/vnd.ms-excel', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet']
        self.file_type_rar = ['application/x-rar-compressed', 'application/x-7z-compressed', 'application/vnd.rar']

        self.file_path_txt = 'txt'
        self.file_path_img = 'img'
        self.file_path_pdf = 'pdf'
        self.file_path_doc = 'doc'
        self.file_path_xls = 'xls'
        self.file_path_rar = 'rar'

    def put(self, key, data):
        self.bucket.put_object(key=key, data=data)

    def get(self, key):
        data = self.bucket.get_object(key).read()
        return data.decode('utf-8')

    def delete(self, key):
        self.bucket.delete_object(key)

    def url(self, key):
        url = f"http://{self.conf.OSS_CONF['bucket']}.{self.conf.OSS_CONF['endpoint']}/{key}"
        return url

    def sign_url(self, key, expires):
        url = self.bucket.sign_url(method='GET', key=key, expires=expires, slash_safe=True)
        return url

    def upload(self, file):
        if not file:
            raise Exception('文件上传失败')
        path = ''
        if file.content_type in self.file_type_txt:
            path += self.file_path_txt
        elif file.content_type in self.file_type_img:
            path += self.file_path_img
        elif file.content_type in self.file_type_pdf:
            path += self.file_path_pdf
        elif file.content_type in self.file_type_doc:
            path += self.file_path_doc
        elif file.content_type in self.file_type_xls:
            path += self.file_path_xls
        elif file.content_type in self.file_type_rar:
            path += self.file_path_rar
        else:
            raise Exception('不支持该文件类型')

        data = file.read()
        file_md5 = hashlib.md5(data).hexdigest()
        for i in range(0, 4):
            path += '/' + file_md5[i*2:i*2+2]
        ext = file.filename.split('.')[-1]
        key = f'{path}/{file_md5}.{ext}'
        self.put(key, data)
        return key



# -*- coding: utf-8 -*-

from app.module.base import BaseController
from app.helper.file import File


class FileController(BaseController):

    def upload(self):
        file = self.file('file')
        f = File()
        path = f.upload(file)

        resp = dict()
        resp['path'] = path
        return self.ok(resp)



# -*- coding: utf-8 -*-

import logging
from importlib import import_module
from flask import redirect
from app.helper.functions import error
from app.helper.decorator import time_cost, assign_connection


class Router(object):

    def __init__(self, app, subdomain=None, module=None):
        self.app = app
        self.subdomain = subdomain
        self.module = module

        @self.app.route('/<path:path>', methods=['GET', 'POST'], endpoint=module, subdomain=subdomain)
        @time_cost
        @assign_connection
        def default(path):
            arr = path.split('/')
            action = arr[-1]
            arr.pop()
            controller = arr[-1]
            arr.pop()
            if module:
                module_arr = module.split('/')
                __module = '.'.join(module_arr + arr)
            else:
                __module = '.'.join(arr)
            return self.dispatcher(__module, controller, action)

        # handle the favicon request
        if self.module:
            favicon_endpoint = module + '/favicon'
        else:
            favicon_endpoint = 'favicon'

        @app.route('/favicon.ico', methods=['GET', 'POST'], endpoint=favicon_endpoint, subdomain=subdomain)
        def favicon():
            return redirect('/static/favicon.ico')

    def add(self, path, handler, methods=None):
        if methods is None:
            methods = ['GET', 'POST']

        array = handler.split('/')
        if self.module:
            array.insert(0, self.module)
        handler = '/'.join(array)

        @self.app.route(path, methods=methods, endpoint=handler, subdomain=self.subdomain)
        @time_cost
        @assign_connection
        def route(**kwargs):
            arr = handler.split('/')
            action = arr[-1]
            arr.pop()
            controller = arr[-1]
            arr.pop()
            module = '.'.join(arr)
            return self.dispatcher(module, controller, action, **kwargs)

    @staticmethod
    def dispatcher(module='', controller='index', action='index', **kwargs):
        try:
            ctrl_list = [ctrl for ctrl in controller.split('-')]
            ctrl_file = '_'.join([ctrl for ctrl in ctrl_list])
            if module:
                module = module.replace('-', '_')
                module_name = f'app.module.{module}.{ctrl_file}'
            else:
                module_name = f'app.module.{ctrl_file}'
            m = import_module(module_name)
            ctrl_class = ''.join(ctrl.capitalize() for ctrl in ctrl_list)
            c = getattr(m, ctrl_class + 'Controller')
            method = action.replace('-', '_')
            f = getattr(c(), method)
            return f(**kwargs)
        except Exception as e:
            logging.error(str(e))
            return error(str(e))



# -*- coding: utf-8 -*-

import os
import warnings
from datetime import timedelta

warnings.filterwarnings('ignore')


class Config(object):
    ENV = 'development'
    DEBUG = False
    TESTING = False
    HOST = '127.0.0.1'
    PORT = 80
    # SERVER_NAME = 'hallo.com'
    BASE_PATH = os.path.abspath(os.path.dirname(__file__))

    # session
    SECRET_KEY = ''
    PERMANENT_SESSION_LIFETIME = timedelta(hours=1)

    # file upload
    MAX_CONTENT_LENGTH = 8 * 1024 * 1024  # 8M

    # mysql
    MYSQL_POOL_SIZE = 1
    MYSQL_CONF = dict(
        host='127.0.0.1',
        port=3306,
        user='root',
        password='password',
        database='hallo',
        charset='utf8'
    )

    # redis
    REDIS_URL = 'redis://127.0.0.1:6379/0'

    # cache
    CACHE_CONF = [
        '127.0.0.1:11211'
    ]

    # oss
    OSS_CONF = dict(
        endpoint='',
        bucket='',
        access_key_id='',
        access_key_secret=''
    )


class DevelopmentConfig(Config):
    DEBUG = True


class ProductionConfig(Config):
    ENV = 'production'
    MYSQL_MAX_CONN = 8
    MYSQL_CONF = dict(
        host='127.0.0.1',
        port=3306,
        user='root',
        password='password',
        database='hallo',
        charset='utf8'
    )


class TestingConfig(Config):
    ENV = 'testing'
    TESTING = True


config = dict(
    dev=DevelopmentConfig,
    prod=ProductionConfig,
    test=TestingConfig
)



# -*- coding: utf-8 -*-

from app.module.base import BaseController
from app.module.user import User
from app.helper.decorator import check_token


class UserController(BaseController):

    def login(self):
        username = self.post('username')
        password = self.post('password')
        user = User.find().where(username=username).one()
        if not user:
            return self.error('用户不存在')
        if user.password != password:
            return self.error('密码错误')
        token = user.login()
        resp = dict()
        resp['token'] = token
        return self.ok(resp)

    @check_token
    def logout(self):
        self.user.logout()
        return self.ok('退出成功')

    @check_token
    def info(self):
        user = self.user

        resp = dict()
        resp['userid'] = user.id
        resp['username'] = user.username
        resp['time'] = user.time
        return self.ok(resp)



# -*- coding: utf-8 -*-

from flask import Flask
from flask_redis import FlaskRedis
from config import config
from app.helper.functions import create_log
from pymyorm.connection_pool import ConnectionPool
import os
import memcache

config_name = os.environ.get('FLASK_CONFIG', 'dev')

root_path = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
template_folder = os.path.join(root_path, 'templates')
static_folder = os.path.join(root_path, 'static')

app = Flask(import_name=__name__, template_folder=template_folder, static_folder=static_folder)
app.config.from_object(config[config_name])

# logging
base_path = app.config['BASE_PATH']
log_dir = os.path.join(base_path, 'log')
if not os.path.exists(log_dir):
    os.mkdir(log_dir)
log_file = os.path.join(log_dir, 'app.log')
create_log(log_file=log_file)

app.logger.info('flask app init')

debug = app.config.get('DEBUG', False)

# mysql
mysql_conf = app.config.get('MYSQL_CONF')
if mysql_conf:
    pool = ConnectionPool()
    mysql_pool_size = app.config.get('MYSQL_POOL_SIZE', 1)
    pool.size(size=mysql_pool_size)
    mysql_conf['debug'] = debug
    pool.create(**mysql_conf)

# redis
app.config['redis'] = FlaskRedis(app)

# cache
cache_conf = app.config.get('CACHE_CONF')
if cache_conf:
    app.config['cache'] = memcache.Client(cache_conf, debug=debug)

from app.router import *



# -*- coding: utf-8 -*-

from app.module.base import BaseController
from flask import session
from app.models.user import User
from app.helper.decorator import check_login


class UserController(BaseController):

    def login(self):
        username = self.post('username')
        password = self.post('password')
        user = User.find().where(username=username).one()
        if not user:
            return self.error('用户不存在')
        if user.password != password:
            return self.error('密码错误')
        session['userid'] = user.id
        return self.ok('登录成功')

    @check_login
    def logout(self):
        session.pop('userid')
        return self.ok('退出成功')

    @check_login
    def info(self):
        userid = session['userid']
        user = User.find().where(id=userid).one()
        if not user:
            return self.error('用户不存在')

        resp = dict()
        resp['userid'] = user.id
        resp['username'] = user.username
        resp['time'] = user.time
        return self.ok(resp)



# -*- coding: utf-8 -*-

import os.path
import shutil
import sys


class Command(object):

    def run(self):
        cmd = 'help'
        if len(sys.argv) > 1:
            cmd = sys.argv[1]
        if cmd in ['help', '--help', '-h']:
            self.help()
        elif cmd in ['version', '--version', '-v']:
            self.version()
        elif cmd == 'create':
            self.create()
        elif cmd == 'install':
            self.install()
        else:
            self.notice(f'{cmd} is not available, Type `hallo help` to see all available commands')

    @staticmethod
    def help():
        print('Available commands:')
        command_list = ['help', 'version', 'create', 'install']
        for command in command_list:
            print(command)

    @staticmethod
    def version():
        from hallo import VERSION
        version = '.'.join(str(x) for x in VERSION)
        print(version)

    def create(self):
        if len(sys.argv) < 3:
            self.notice('project name is missing, Type `hallo create project_name`')
            return
        project_name = sys.argv[2]
        project_dir = os.path.join(os.getcwd(), project_name)
        frame_dir = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'frame')
        self.copy_dir(frame_dir, project_dir)
        os.makedirs(os.path.join(project_dir, 'log'))
        os.remove(os.path.join(project_dir, '__init__.py'))

    def install(self):
        base_path = os.getcwd()
        project = base_path.split('/')[-1]
        tpl_install_shell = os.path.join(os.path.dirname(__file__), 'install.sh')
        fp = open(tpl_install_shell, 'r', encoding='utf-8')
        data = fp.read()
        fp.close()

        prj_install_shell = os.path.join(base_path, 'install.sh')
        fp = open(prj_install_shell, 'w', encoding='utf-8')
        data_map = dict(
            base_path=base_path,
            project=project
        )
        fp.write(data % data_map)
        fp.close()

    def notice(self, text):
        print(f'\033[0;31m{text}\033[0m')

    def copy_dir(self, from_file, to_file):
        if not os.path.exists(to_file):
            os.makedirs(to_file)
        files = os.listdir(from_file)
        for f in files:
            if os.path.isdir(f'{from_file}/{f}'):
                self.copy_dir(f'{from_file}/{f}', f'{to_file}/{f}')
            else:
                shutil.copy(f'{from_file}/{f}', f'{to_file}/{f}')



# -*- coding: utf-8 -*-

import functools
import time
import logging
from flask import request, session, current_app as app
from app.helper.functions import error
from pymyorm.local import local
from pymyorm.connection_pool import ConnectionPool


# assign one connection to the request
def assign_connection(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        pool = ConnectionPool()
        try:
            local.conn = pool.get()
            return func(*args, **kwargs)
        except Exception as e:
            raise e
        finally:
            # don't forget to put connection into pool
            pool.put(local.conn)
    return wrapper


def check_login(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        userid = session.get('userid')
        if not userid:
            return error('login is required', 401)
        else:
            return func(*args, **kwargs)
    return wrapper


def check_token(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        token = request.headers.get('Token')
        if not token:
            return error('token is missing', 401)
        if not app.config['redis'].get(token):
            return error('token is expired', 401)
        else:
            return func(*args, **kwargs)
    return wrapper


def time_cost(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        start = time.time()
        resp = func(*args, **kwargs)
        end = time.time()
        cost = round(end - start, 3)
        logging.info(f'{request.path} time cost={cost}s')
        return resp
    return wrapper


def retry(num=1, seconds=0):
    def outer(func):
        @functools.wraps(func)
        def inner(*args, **kwargs):
            for i in range(0, num):
                try:
                    resp = func(*args, **kwargs)
                    if resp['code'] == 0:
                        return resp
                    if seconds > 0:
                        time.sleep(seconds)
                except Exception as e:
                    logging.error(str(e))
            return error(f'retry {num} times and failed')
        return inner
    return outer



# -*- coding: utf-8 -*-

from flask import request, render_template, current_app as app
from app.models.user import User
import os
import math


class BaseController(object):

    OK = 0
    ERROR = 1

    def __init__(self):
        self.conf = app.config
        self.redis = app.config.get('redis')
        self.cache = app.config.get('cache')
        self.page = 0
        self.offset = 0
        self.limit = 0

    @staticmethod
    def ok(data=None, code=OK):
        if data is None:
            data = 'ok'
        return dict(code=code, data=data)

    @staticmethod
    def error(data=None, code=ERROR):
        if data is None:
            data = 'error'
        return dict(code=code, data=data)

    @staticmethod
    def get(name):
        return request.args.get(name)

    @staticmethod
    def post(name):
        return request.json.get(name)

    @staticmethod
    def file(name):
        return request.files.get(name)

    @staticmethod
    def header(name):
        return request.headers.get(name)

    def render(self, template, data=None):
        base_path = self.conf.get('BASE_PATH')
        filename = os.path.join(base_path, 'templates', template)
        if not os.path.exists(filename):
            raise Exception(f'file not found: {filename}')
        if data is None:
            return render_template(template)
        else:
            return render_template(template, **data)

    def init_page(self):
        self.page = int(request.args.get('page', 1))
        self.limit = int(request.args.get('limit', 10))
        self.offset = (self.page - 1) * self.limit

    def resp_page(self, all, total):
        total_page = math.ceil(total / self.limit)
        data = dict(
            list=all,
            page=self.page,
            limit=self.limit,
            total=total,
            total_page=total_page
        )
        return self.ok(data)

    @property
    def userid(self):
        token = self.header('Token')
        userid = self.redis.get(token)
        if not userid:
            return None
        return str(userid, 'UTF-8')

    @property
    def user(self):
        userid = self.userid
        if not userid:
            return None
        user = User.find().where(id=userid).one()
        return user



# -*- coding: utf-8 -*-

import os.path
from app.console.base import BaseCommand
from pymyorm.database import Database
import logging


class TableCommand(BaseCommand):

    log_file_name = 'table.log'

    @staticmethod
    def show():
        tables = Database.tables()
        for table in tables:
            logging.info(table)

    def build(self):
        base_path = self.base_path
        sql_path = os.path.join(base_path, 'sql')
        filenames = os.listdir(sql_path)
        for filename in filenames:
            table = filename.split('.')[0]
            exists = Database.exists(table)
            if exists:
                logging.warning(f'table {table} exists')
            else:
                file = os.path.join(self.base_path, 'sql', filename)
                logging.info(f'create table {table}')
                fp = open(file, 'r', encoding='utf-8')
                sql = fp.read()
                try:
                    Database.execute(sql)
                except Exception as e:
                    logging.error(str(e))

    def model(self):
        tables = Database.tables()
        for table in tables:
            model = f'{self.base_path}/app/models/{table}.py'
            if os.path.exists(model):
                logging.warning(f'model {table} exists')
            else:
                logging.info(f'create model {table}')
                Database.reflect(table=table, model=model)


if __name__ == '__main__':
    cmd = TableCommand()
    cmd.build()



from pymyorm.model import Model
from flask import request, current_app as app
from werkzeug.security import generate_password_hash, check_password_hash
import time
import hashlib


class User(Model):
    tablename = 'user'

    default_password = '123456'

    def set_password(self, password=None):
        if password is None:
            password = self.default_password
        self.password = generate_password_hash(password)

    def check_password(self, password):
        return check_password_hash(self.password, password)

    def login(self, expired=None):
        token = self.create_token()
        if expired is None:
            lifetime = app.config.get('PERMANENT_SESSION_LIFETIME')
            if lifetime:
                expired = lifetime.seconds
            else:
                expired = 3600
        app.config['redis'].set(name=token, value=self.id, ex=expired)
        return token

    @staticmethod
    def logout():
        token = request.headers.get('Token')
        app.config['redis'].delete(token)

    def create_token(self):
        md5 = hashlib.md5()
        md5.update(str(time.time()).encode('utf-8'))
        md5.update(str(self.id).encode('utf-8'))
        return md5.hexdigest()



# -*- coding: utf-8 -*-

import sys
from app.helper.console import Console


def main():
    if len(sys.argv) < 2:
        print('参数格式错误!')
        print('正确格式参考: python console.py module/action [--param=value]...')
        return
    command = sys.argv[1]
    args = sys.argv
    args.pop(0)
    args.pop(0)
    params = dict()
    for arg in args:
        if not arg.startswith('--'):
            print('参数格式错误: --param=value')
            return
        arg = arg[2:]
        if not arg.index('='):
            print('参数格式错误: --param=value')
            return
        k, v = arg.split('=')
        params[k] = v
    Console.run(command, **params)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

import os
import hashlib
from config import config


class File(object):

    def __init__(self):
        config_name = os.environ.get('FLASK_CONFIG', 'dev')
        conf = config[config_name]
        self.base_path = conf.BASE_PATH

        # 设置允许上传的文件类型
        self.file_type_txt = ['text/plain']
        self.file_type_img = ['image/jpeg', 'image/bmp', 'image/png', 'image/gif']
        self.file_type_pdf = ['application/pdf']
        self.file_type_doc = ['application/msword', 'application/vnd.openxmlformats-officedocument.wordprocessingml.document']
        self.file_type_xls = ['application/vnd.ms-excel', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet']
        self.file_type_rar = ['application/x-rar-compressed', 'application/x-7z-compressed', 'application/vnd.rar']

        self.file_path_txt = 'txt'
        self.file_path_img = 'img'
        self.file_path_pdf = 'pdf'
        self.file_path_doc = 'doc'
        self.file_path_xls = 'xls'
        self.file_path_rar = 'rar'

    def upload(self, file):
        if not file:
            raise Exception('文件上传失败')
        path = 'static/upload/'
        if file.content_type in self.file_type_txt:
            path += self.file_path_txt
        elif file.content_type in self.file_type_img:
            path += self.file_path_img
        elif file.content_type in self.file_type_pdf:
            path += self.file_path_pdf
        elif file.content_type in self.file_type_doc:
            path += self.file_path_doc
        elif file.content_type in self.file_type_xls:
            path += self.file_path_xls
        elif file.content_type in self.file_type_rar:
            path += self.file_path_rar
        else:
            raise Exception('不支持该文件类型')

        data = file.read()
        file.seek(0)
        file_md5 = hashlib.md5(data).hexdigest()
        for i in range(0, 4):
            path += '/' + file_md5[i*2:i*2+2]
        disk_path = f'{self.base_path}/{path}'
        if not os.path.exists(disk_path):
            os.makedirs(disk_path)
        ext = file.filename.split('.')[-1]
        file_path = f'{path}/{file_md5}.{ext}'
        real_path = f'{disk_path}/{file_md5}.{ext}'
        file.save(real_path)
        return file_path



# -*- coding: utf-8 -*-

from app.module.base import BaseController
from app.helper.oss import Oss


class OssController(BaseController):

    def upload(self):
        file = self.file('file')
        oss = Oss()
        key = oss.upload(file)

        resp = dict()
        resp['key'] = key
        resp['url'] = oss.url(key)
        resp['sign_url'] = oss.sign_url(key=key, expires=3600)
        return self.ok(resp)



# -*- coding: utf-8 -*-

from logging.config import dictConfig


def ok(data=None):
    if data is None:
        data = 'ok'
    return dict(code=0, data=data)


def error(data=None, code=None):
    if data is None:
        data = 'error'
    if code is None:
        code = 1
    return dict(code=code, data=data)


def create_log(log_file):
    dictConfig({
        'version': 1,
        'formatters': {
            'default': {
                'format': '%(asctime)s %(levelname)s [%(filename)s:%(lineno)s]: %(message)s',
                'datefmt': '%Y-%m-%d %H:%M:%S'
            }
        },
        'handlers': {
            'wsgi': {
                'class': 'logging.StreamHandler',
                'stream': 'ext://flask.logging.wsgi_errors_stream',
                'formatter': 'default'
            },
            'message': {
                'class': 'logging.handlers.RotatingFileHandler',
                'formatter': 'default',
                'filename': log_file,
                'maxBytes': 1024 * 1024,
                'backupCount': 10,
                'encoding': 'utf-8'
            }
        },
        'root': {
            'level': 'INFO',
            'handlers': ['wsgi', 'message']
        }
    })
